@@ -137,8 +137,7 @@ class Base(object):
     def schemas(self):
         """Get the database schema names."""
         if self.__schemas is None:
-            insp = sa.engine.reflection.Inspector.from_engine(self.engine)
-            self.__schemas = insp.get_schema_names()
+            self.__schemas = sa.inspect(self.engine).get_schema_names()
             for schema in BUILTIN_SCHEMAS:
                 if schema in self.__schemas:
                     self.__schemas.remove(schema)
@@ -156,6 +155,22 @@ class Base(object):
         metadata = self.__metadata[schema]
         return metadata.tables.keys()
 
+    def _get_schema(self, schema, table):
+        pairs = table.split('.')
+        if len(pairs) == 2:
+            return pairs[0], pairs[1]
+        if len(pairs) == 1:
+            return schema, pairs[0]
+        raise ValueError(
+            f'Invalid definition {table} for schema: {schema}'
+        )
+
+    def _get_table(self, schema, table):
+        """Get fully qualified table name."""
+        if not table.startswith(f'{schema}.'):
+            return f'{schema}.{table}'
+        return table
+
     def truncate_table(self, table, schema=SCHEMA):
         """Truncate a table.
 
@@ -168,10 +183,8 @@ class Base(object):
             schema (str): The database schema
 
         """
-        if not table.startswith(f'{schema}.'):
-            table = f'{schema}.{table}'
-        pairs = table.split('.')
-        schema, table = pairs[0], pairs[1]
+        table = self._get_table(schema, table)
+        schema, table = self._get_schema(schema, table)
         logger.debug(f'Truncating table: {schema}.{table}')
         query = f'TRUNCATE TABLE "{schema}"."{table}" CASCADE'
         self.execute(query)
@@ -209,13 +222,14 @@ class Base(object):
         statement = statement.select_from(
             sa.text('PG_REPLICATION_SLOTS')
         )
-        filters = [
-            sa.column('slot_name') == slot_name,
-            sa.column('slot_type') == slot_type,
-            sa.column('plugin') == plugin,
-        ]
-        statement = statement.where(sa.and_(*filters))
-        return self.query_all(statement)
+        statement = statement.where(
+            sa.and_(*[
+                sa.column('slot_name') == slot_name,
+                sa.column('slot_type') == slot_type,
+                sa.column('plugin') == plugin,
+            ])
+        )
+        return self.query(statement)
 
     def create_replication_slot(self, slot_name):
         """
@@ -259,7 +273,8 @@ class Base(object):
         slot_name,
         txmin=None,
         txmax=None,
-        upto_nchanges=1,
+        upto_lsn=None,
+        upto_nchanges=None,
     ):
         """
         Get/Consume changes from a logical replication slot.
@@ -276,7 +291,7 @@ class Base(object):
         statement = statement.select_from(
             sa.func.PG_LOGICAL_SLOT_GET_CHANGES(
                 slot_name,
-                None,
+                upto_lsn,
                 upto_nchanges,
             )
         )
@@ -298,14 +313,15 @@ class Base(object):
             statement = statement.where(sa.and_(*filters))
         if self.verbose:
             compiled_query(statement, 'logical_slot_get_changes')
-        return self.query_all(statement)
+        return self.query(statement)
 
     def logical_slot_peek_changes(
         self,
         slot_name,
         txmin=None,
         txmax=None,
-        upto_nchanges=1,
+        upto_lsn=None,
+        upto_nchanges=None,
     ):
         """
         Peek a logical replication slot without consuming changes.
@@ -317,7 +333,7 @@ class Base(object):
         statement = sa.select(columns).select_from(
             sa.func.PG_LOGICAL_SLOT_PEEK_CHANGES(
                 slot_name,
-                None,
+                upto_lsn,
                 upto_nchanges,
             )
         )
@@ -339,20 +355,16 @@ class Base(object):
             statement = statement.where(sa.and_(*filters))
         if self.verbose:
             compiled_query(statement, 'logical_slot_peek_changes')
-        return self.query_all(statement)
+        return self.query(statement)
 
     # Triggers...
-    def create_triggers(self, database, schema, tables=None, drop=False):
+    def create_triggers(self, database, schema, tables=None):
         """Create a database triggers."""
-        if drop:
-            self.drop_triggers(database, schema)
         self.execute(CREATE_TRIGGER_TEMPLATE)
         queries = []
-        insp = sa.engine.reflection.Inspector.from_engine(self.engine)
-        views = insp.get_view_names(schema)
+        views = sa.inspect(self.engine).get_view_names(schema)
         for table in self.tables(schema):
-            pairs = table.split('.')
-            schema, table = pairs[0], pairs[1]
+            schema, table = self._get_schema(schema, table)
             if (tables and table not in tables) or (table in views):
                 continue
             logger.debug(f'Creating trigger on table: {schema}.{table}')
@@ -360,6 +372,10 @@ class Base(object):
                 ('notify', 'ROW', ['INSERT', 'UPDATE', 'DELETE']),
                 ('truncate', 'STATEMENT', ['TRUNCATE']),
             ]:
+
+                if self.exist_trigger(f'{table}_{name}', f'{schema}.{table}'):
+                    continue
+
                 queries.append(
                     f'CREATE TRIGGER {table}_{name} '
                     f'AFTER {" OR ".join(tg_op)} ON "{schema}"."{table}" '
@@ -372,8 +388,7 @@ class Base(object):
     def drop_triggers(self, database, schema, tables=None):
         """Drop all pgsync defined triggers in database."""
         for table in self.tables(schema):
-            pairs = table.split('.')
-            schema, table = pairs[0], pairs[1]
+            schema, table = self._get_schema(schema, table)
             if tables and table not in tables:
                 continue
             logger.debug(f'Dropping trigger on table: {schema}.{table}')
@@ -387,8 +402,7 @@ class Base(object):
     def disable_triggers(self, database, schema):
         """Disable all pgsync defined triggers in database."""
         for table in self.tables(schema):
-            pairs = table.split('.')
-            schema, table = pairs[0], pairs[1]
+            schema, table = self._get_schema(schema, table)
             logger.debug(f'Disabling trigger on table: {schema}.{table}')
             for name in ('notify', 'truncate'):
                 query = (
@@ -400,8 +414,7 @@ class Base(object):
     def enable_triggers(self, database, schema):
         """Enable all pgsync defined triggers in database."""
         for table in self.tables(schema):
-            pairs = table.split('.')
-            schema, table = pairs[0], pairs[1]
+            schema, table = self._get_schema(schema, table)
             logger.debug(f'Enabling trigger on table: {schema}.{table}')
             for name in ('notify', 'truncate'):
                 query = (
@@ -410,6 +423,24 @@ class Base(object):
                 )
                 self.execute(query)
 
+    def exist_trigger(self, tgname, tgrelid):
+        """Check if the given trigger exists on table.
+
+        Args:
+            tgname (str): The trigger name
+            tgrelid (str): The table name
+
+        Returns:
+            True if exists, False otherwise.
+        """
+        statement = sa.text(
+            f"SELECT tgname FROM pg_trigger "
+            f"WHERE NOT tgisinternal "
+            f"AND tgrelid = '{tgrelid}'::REGCLASS "
+            f"AND tgname = '{tgname}' "
+        )
+        return self.query_one(statement) is not None
+
     def execute(self, query, values=None, options=None):
         """Execute a query command."""
         conn = self.__engine.connect()
@@ -531,6 +562,8 @@ class Base(object):
                     break
 
                 key = match.groupdict().get('key')
+                if key:
+                    key = key.replace('"', '')
                 value = match.groupdict().get('value')
                 type_ = match.groupdict().get('type')
 
@@ -614,7 +647,7 @@ class Base(object):
             raise
         return row
 
-    def query_all(self, query):
+    def query(self, query):
         conn = self.__engine.connect()
         try:
             rows = conn.execute(query).fetchall()
@@ -627,6 +660,7 @@ class Base(object):
     def query_yield(self, query, chunk_size=None):
         chunk_size = chunk_size or QUERY_CHUNK_SIZE
         with self.__engine.connect() as conn:
+            conn = conn.execution_options(stream_results=True)
             result = conn.execute(query)
             while True:
                 chunk = result.fetchmany(chunk_size)
@@ -637,7 +671,10 @@ class Base(object):
 
     def query_count(self, query):
         with self.__engine.connect() as conn:
-            return conn.execute(query).rowcount
+            query = query.original.with_only_columns(
+                [sa.func.count()]
+            ).order_by(None)
+            return conn.execute(query).scalar()
 
 
 # helper methods
@@ -695,7 +732,7 @@ def _get_foreign_keys(model_a, model_b):
     return foreign_keys
 
 
-def get_foreign_keys(model_a, model_b):
+def get_foreign_keys(node_a, node_b):
     """Return dict of single foreign key with multiple columns.
 
     e.g:
@@ -707,9 +744,31 @@ def get_foreign_keys(model_a, model_b):
     column_1, column_2, column_N are of type ForeignKeyContraint
     """
     foreign_keys = {}
-    _foreign_keys = _get_foreign_keys(model_a, model_b)
-    for table, columns in _foreign_keys.items():
-        foreign_keys[table] = sorted([column.name for column in columns])
+    # if either offers a foreign_key via relationship, use it!
+    if (
+        node_a.relationship.foreign_key.parent or
+        node_b.relationship.foreign_key.parent
+    ):
+        if node_a.relationship.foreign_key.parent:
+            foreign_keys[node_a.parent.name] = sorted(
+                node_a.relationship.foreign_key.parent
+            )
+            foreign_keys[node_a.name] = sorted(
+                node_a.relationship.foreign_key.child
+            )
+        if node_b.relationship.foreign_key.parent:
+            foreign_keys[node_b.parent.name] = sorted(
+                node_b.relationship.foreign_key.parent
+            )
+            foreign_keys[node_b.name] = sorted(
+                node_b.relationship.foreign_key.child
+            )
+    else:
+        for table, columns in _get_foreign_keys(
+            node_a.model,
+            node_b.model,
+        ).items():
+            foreign_keys[table] = sorted([column.name for column in columns])
     return foreign_keys
 
 
@@ -771,10 +830,11 @@ def pg_execute(engine, query, values=None, options=None):
         raise
 
 
-def create_schema(base, engine):
+def create_schema(engine, schema):
     """Create database schema."""
-    base.metadata.drop_all(engine)
-    base.metadata.create_all(engine)
+    if schema == SCHEMA:
+        return
+    engine.execute(sa.schema.CreateSchema(schema))
 
 
 def create_database(database, echo=False):
@@ -827,7 +887,7 @@ def refresh_materialized_view(database, view, echo=False):
 
 def drop_materialized_view(database, view, echo=False):
     """Drop a materialized database view."""
-    logger.debug(f'Drop materialized view: {view}')
+    logger.debug(f'Dropping materialized view: {view}')
     engine = pg_engine(database=database, echo=echo)
     pg_execute(engine, f'DROP MATERIALIZED VIEW IF EXISTS {view}')
     logger.debug(f'Dropped materialized view: {view}')
