@@ -4,6 +4,7 @@ import sqlalchemy as sa
 from .base import compiled_query, get_foreign_keys, get_primary_keys
 from .constants import OBJECT, ONE_TO_MANY, ONE_TO_ONE, SCALAR
 from .exc import FetchColumnForeignKeysError
+from .node import node_from_table
 
 
 class QueryBuilder(object):
@@ -24,35 +25,35 @@ class QueryBuilder(object):
 
     def _get_foreign_keys(self, model_a, model_b):
 
-        if model_a.through_tables or model_b.through_tables:
+        if (
+            model_a.relationship.through_tables or
+            model_b.relationship.through_tables
+        ):
 
             if model_a.through_tables:
-                through_tables = model_a.through_tables[0]
+                through_tables = model_a.relationship.through_tables[0]
 
             if model_b.through_tables:
-                through_tables = model_b.through_tables[0]
+                through_tables = model_b.relationship.through_tables[0]
                 model_a, model_b = model_b, model_a
 
-            through = self.base.model(through_tables, model_a.schema)
-            foreign_keys = get_foreign_keys(
-                model_a.model,
-                through,
+            through = node_from_table(
+                self.base,
+                through_tables,
+                model_a.schema,
             )
-            for key, value in get_foreign_keys(
-                through,
-                model_b.model,
-            ).items():
+            foreign_keys = get_foreign_keys(model_a, through)
+            for key, value in get_foreign_keys(through, model_b).items():
                 if key in foreign_keys:
                     foreign_keys[key].extend(value)
                     continue
                 foreign_keys[key] = value
 
+            return foreign_keys
+
         else:
-            return get_foreign_keys(
-                model_a.model,
-                model_b.model,
-            )
-        return foreign_keys
+
+            return get_foreign_keys(model_a, model_b)
 
     def _get_column_foreign_keys(
         self,
@@ -73,7 +74,12 @@ class QueryBuilder(object):
             columns = ['column_a', 'column_b']
             returns ['column_a', 'column_b']
         """
-        column_names = [column.name for column in columns]
+        # TODO: normalize this elsewhere
+        try:
+            column_names = [column.name for column in columns]
+        except AttributeError:
+            column_names = [column for column in columns]
+
         if table is None:
             for table, cols in foreign_keys.items():
                 if set(cols).issubset(set(column_names)):
@@ -100,12 +106,12 @@ class QueryBuilder(object):
                 node.table,
                 params,
             ),
-            sa.dialects.postgresql.JSONB
+            sa.dialects.postgresql.JSONB,
         )
         for child in node.children:
             if (
-                not child.parent.through_tables and
-                child.parent.relationship_type == ONE_TO_MANY
+                not child.parent.relationship.through_tables and
+                child.parent.relationship.type == ONE_TO_MANY
             ):
                 row = row.concat(
                     sa.cast(
@@ -159,7 +165,7 @@ class QueryBuilder(object):
 
             onclause = []
 
-            if child.through_tables:
+            if child.relationship.through_tables:
 
                 child.parent.columns.extend(
                     [
@@ -168,19 +174,17 @@ class QueryBuilder(object):
                     ]
                 )
 
-                through = self.base.model(
-                    child.through_tables[0],
+                through = node_from_table(
+                    self.base,
+                    child.relationship.through_tables[0],
                     child.schema,
                 )
-                foreign_keys = get_foreign_keys(
-                    child.parent.model,
-                    through,
-                )
+                foreign_keys = get_foreign_keys(child.parent, through)
 
                 left_foreign_keys = self._get_column_foreign_keys(
                     child._subquery.columns,
                     foreign_keys,
-                    table=through.original.name,
+                    table=through.name,
                     schema=child.schema,
                 )
                 right_foreign_keys = self._get_column_foreign_keys(
@@ -209,10 +213,7 @@ class QueryBuilder(object):
                     ]
                 )
 
-                foreign_keys = get_foreign_keys(
-                    node.model,
-                    child.model,
-                )
+                foreign_keys = get_foreign_keys(node, child)
                 left_foreign_keys = self._get_column_foreign_keys(
                     child._subquery.columns,
                     foreign_keys,
@@ -290,13 +291,14 @@ class QueryBuilder(object):
 
     def _through(self, node):  # noqa: C901
 
-        through = self.base.model(node.through_tables[0], node.schema)
-        foreign_keys = get_foreign_keys(node.model, through)
+        through = node_from_table(
+            self.base,
+            node.relationship.through_tables[0],
+            node.schema,
+        )
+        foreign_keys = get_foreign_keys(node, through)
 
-        for key, value in get_foreign_keys(
-            through,
-            node.parent.model
-        ).items():
+        for key, value in get_foreign_keys(through, node.parent).items():
             if key in foreign_keys:
                 foreign_keys[key].extend(value)
                 continue
@@ -332,15 +334,15 @@ class QueryBuilder(object):
 
         columns = [_keys]
         # We need to include through keys and the actual keys
-        if node.relationship_variant == SCALAR:
+        if node.relationship.variant == SCALAR:
             columns.append(
                 node.columns[1].label(
                     'anon'
                 )
             )
-        elif node.relationship_variant == OBJECT:
+        elif node.relationship.variant == OBJECT:
 
-            if node.relationship_type == ONE_TO_ONE:
+            if node.relationship.type == ONE_TO_ONE:
 
                 if not node.children:
                     columns.append(
@@ -355,7 +357,7 @@ class QueryBuilder(object):
                             *node.columns
                         ).label('anon')
                     )
-            elif node.relationship_type == ONE_TO_MANY:
+            elif node.relationship.type == ONE_TO_MANY:
                 columns.append(
                     sa.func.JSON_BUILD_OBJECT(
                         *node.columns
@@ -375,19 +377,17 @@ class QueryBuilder(object):
 
             onclause = []
 
-            if child.through_tables:
+            if child.relationship.through_tables:
 
-                child_through = self.base.model(
-                    child.through_tables[0],
+                child_through = node_from_table(
+                    self.base,
+                    child.relationship.through_tables[0],
                     child.schema,
                 )
-                child_foreign_keys = get_foreign_keys(
-                    child.model,
-                    child_through,
-                )
+                child_foreign_keys = get_foreign_keys(child, child_through)
                 for key, value in get_foreign_keys(
                     child_through,
-                    child.parent.model,
+                    child.parent,
                 ).items():
                     if key in child_foreign_keys:
                         child_foreign_keys[key].extend(value)
@@ -397,24 +397,19 @@ class QueryBuilder(object):
                 left_foreign_keys = self._get_column_foreign_keys(
                     child._subquery.columns,
                     child_foreign_keys,
-                    table=child_through.original.name,
+                    table=child_through.table,
                     schema=child.schema,
                 )
 
             else:
 
-                child_foreign_keys = get_foreign_keys(
-                    child.parent.model,
-                    child.model,
-                )
+                child_foreign_keys = get_foreign_keys(child.parent, child)
                 left_foreign_keys = self._get_column_foreign_keys(
                     child._subquery.columns,
                     child_foreign_keys,
                 )
 
-            right_foreign_keys = child_foreign_keys[
-                f'{child.parent.schema}.{child.parent.table}'
-            ]
+            right_foreign_keys = child_foreign_keys[child.parent.name]
 
             for i in range(len(left_foreign_keys)):
                 onclause.append(
@@ -493,7 +488,7 @@ class QueryBuilder(object):
         if self.verbose:
             compiled_query(outer_subquery, 'Outer subquery')
 
-        through_primary_keys = get_primary_keys(through)
+        through_primary_keys = get_primary_keys(through.model)
 
         params = []
         for through_primary_key in through_primary_keys:
@@ -502,7 +497,7 @@ class QueryBuilder(object):
                     str(through_primary_key),
                     sa.func.JSON_BUILD_ARRAY(
                         getattr(
-                            through.c,
+                            through.model.c,
                             through_primary_key,
                         )
                     )
@@ -511,7 +506,7 @@ class QueryBuilder(object):
 
         through_keys = sa.cast(
             sa.func.JSON_BUILD_OBJECT(
-                node.through_tables[0],
+                node.relationship.through_tables[0],
                 sa.func.JSON_BUILD_ARRAY(
                     *params
                 )
@@ -529,11 +524,11 @@ class QueryBuilder(object):
             )
         ).label('_keys')
 
-        left_foreign_keys = foreign_keys[f'{node.schema}.{node.table}']
+        left_foreign_keys = foreign_keys[node.name]
         right_foreign_keys = self._get_column_foreign_keys(
             through.columns,
             foreign_keys,
-            table=through.original.name,
+            table=through.table,
             schema=node.schema,
         )
 
@@ -544,11 +539,11 @@ class QueryBuilder(object):
             ).label(node.label)
         ]
 
-        foreign_keys = get_foreign_keys(node.parent.model, through)
+        foreign_keys = get_foreign_keys(node.parent, through)
 
-        for column in foreign_keys[str(through.original)]:
+        for column in foreign_keys[through.name]:
             columns.append(
-                getattr(through.c, column)
+                getattr(through.model.c, column)
             )
 
         inner_subquery = sa.select(columns)
@@ -564,7 +559,7 @@ class QueryBuilder(object):
                     left_foreign_keys[i],
                 ) ==
                 getattr(
-                    through.c,
+                    through.model.c,
                     right_foreign_keys[i],
                 )
             )
@@ -576,7 +571,7 @@ class QueryBuilder(object):
         if node.table == node.parent.table:
             op = sa.or_
 
-        from_obj = through.join(
+        from_obj = through.model.join(
             outer_subquery,
             onclause=op(*onclause),
             isouter=self.isouter,
@@ -595,10 +590,10 @@ class QueryBuilder(object):
         subquery = subquery.group_by(
             *[
                 getattr(
-                    through.c,
+                    through.model.c,
                     column,
                 ) for column in foreign_keys[
-                    str(through.original)
+                    through.name
                 ]
             ]
         )
@@ -631,7 +626,7 @@ class QueryBuilder(object):
                     ) ==
                     getattr(
                         node.model.c,
-                        foreign_keys[f'{node.schema}.{node.table}'][i],
+                        foreign_keys[node.name][i],
                     )
                 )
 
@@ -679,10 +674,7 @@ class QueryBuilder(object):
                 isouter=self.isouter,
             )
 
-        foreign_keys = get_foreign_keys(
-            node.parent.model,
-            node.model,
-        )
+        foreign_keys = get_foreign_keys(node.parent, node)
 
         foreign_key_columns = self._get_column_foreign_keys(
             node.model.columns,
@@ -713,14 +705,14 @@ class QueryBuilder(object):
                     )
                 ])
 
-        if node.relationship_type == ONE_TO_ONE:
+        if node.relationship.type == ONE_TO_ONE:
             _keys = self._get_child_keys(
                 node,
                 sa.func.JSON_BUILD_OBJECT(
                     *params
                 )
             )
-        elif node.relationship_type == ONE_TO_MANY:
+        elif node.relationship.type == ONE_TO_MANY:
             _keys = self._get_child_keys(
                 node,
                 sa.func.JSON_AGG(
@@ -732,16 +724,16 @@ class QueryBuilder(object):
 
         columns = [_keys]
 
-        if node.relationship_variant == SCALAR:
+        if node.relationship.variant == SCALAR:
             # TODO: Raise exception here if the number of columns > 1
-            if node.relationship_type == ONE_TO_ONE:
+            if node.relationship.type == ONE_TO_ONE:
                 columns.append(
                     getattr(
                         node.model.c,
                         node.columns[0],
                     ).label(node.label)
                 )
-            elif node.relationship_type == ONE_TO_MANY:
+            elif node.relationship.type == ONE_TO_MANY:
                 columns.append(
                     sa.func.JSON_AGG(
                         getattr(
@@ -750,14 +742,14 @@ class QueryBuilder(object):
                         )
                     ).label(node.label)
                 )
-        elif node.relationship_variant == OBJECT:
-            if node.relationship_type == ONE_TO_ONE:
+        elif node.relationship.variant == OBJECT:
+            if node.relationship.type == ONE_TO_ONE:
                 columns.append(
                     sa.func.JSON_BUILD_OBJECT(
                         *node.columns
                     ).label(node.label)
                 )
-            elif node.relationship_type == ONE_TO_MANY:
+            elif node.relationship.type == ONE_TO_MANY:
                 columns.append(
                     sa.func.JSON_AGG(
                         sa.func.JSON_BUILD_OBJECT(
@@ -786,7 +778,7 @@ class QueryBuilder(object):
                 sa.and_(*node._filters)
             )
 
-        if node.relationship_type == ONE_TO_MANY:
+        if node.relationship.type == ONE_TO_MANY:
             node._subquery = node._subquery.group_by(
                 *[
                     getattr(
@@ -809,7 +801,7 @@ class QueryBuilder(object):
             self._root(node)
         else:
             # 2) subquery: these are for children creating their own columns
-            if node.through_tables:
+            if node.relationship.through_tables:
                 self._through(node)
             else:
                 self._non_through(node)
