@@ -19,7 +19,7 @@ POLL_TIMEOUT = env.float('POLL_TIMEOUT', default=0.1)
 # replication slot cleanup interval (in secs)
 REPLICATION_SLOT_CLEANUP_INTERVAL = env.float(
     'REPLICATION_SLOT_CLEANUP_INTERVAL',
-    default=3600.0,
+    default=180.0,
 )
 
 # Elasticsearch:
@@ -46,6 +46,19 @@ ELASTICSEARCH_VERIFY_CERTS = env.bool(
     'ELASTICSEARCH_VERIFY_CERTS',
     default=True,
 )
+# turn on SSL
+ELASTICSEARCH_USE_SSL = env.bool('ELASTICSEARCH_USE_SSL', default=False)
+# don't show warnings about ssl certs verification
+ELASTICSEARCH_SSL_SHOW_WARN = env.bool(
+    'ELASTICSEARCH_SSL_SHOW_WARN',
+    default=False,
+)
+# provide a path to CA certs on disk
+ELASTICSEARCH_CA_CERTS = env.str('ELASTICSEARCH_CA_CERTS', default=None)
+# PEM formatted SSL client certificate
+ELASTICSEARCH_CLIENT_CERT = env.str('ELASTICSEARCH_CLIENT_CERT', default=None)
+# PEM formatted SSL client key
+ELASTICSEARCH_CLIENT_KEY = env.str('ELASTICSEARCH_CLIENT_KEY', default=None)
 
 # Postgres:
 PG_HOST = env.str('PG_HOST', default='localhost')
@@ -64,6 +77,8 @@ REDIS_AUTH = env.str('REDIS_AUTH', default=None)
 REDIS_CHUNK_SIZE = env.int('REDIS_CHUNK_SIZE', default=1000)
 # redis socket connection timeout
 REDIS_SOCKET_TIMEOUT = env.int('REDIS_SOCKET_TIMEOUT', default=5)
+# redis poll interval (in secs)
+REDIS_POLL_INTERVAL = env.float('REDIS_POLL_INTERVAL', default=0.01)
 
 
 # Logging:
